/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.msm;

import java.io.*;
import java.util.*;
import java.util.jar.*;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.file.FileNode;
import org.greenstone.gatherer.util.Utility;
/** Attempts to locate and import any metadata already attatched to files about to be imported. This entire function is part of a critical file copy and so must be fast and reliable (or at least recover nicely when things go wrong). Futhermore this process must be aware of concerns such as previous greenstone collections having metadata sets that could be immediately imported, or hierarchy files needed to turn indexes into something more meaningful. We also must allow for the extensibility of this class to include custom metadata objects such as MARK records etc. 
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class ExistingMetadataLoader {
    /** A list of currently loaded metadata parsers. */
    private ArrayList parsers = null;
    /** Constructor.
     * @see org.greenstone.gatherer.msm.MetadataParser
     * @see org.greenstone.gatherer.util.Utility
     */
    public ExistingMetadataLoader() {
	this.parsers = new ArrayList();
	ArrayList load_these = new ArrayList();
	// Find any metadata parsers found in parsers directory
	File file = new File(Utility.BASE_DIR + "classes" + File.separator + "org" + File.separator + "greenstone" + File.separator + "gatherer" + File.separator + "msm" + File.separator + "parsers");
	File children[] = file.listFiles();
	for(int i = 0; children != null && i < children.length; i++) {
	    String name = children[i].getName();
	    if(name.endsWith(".class") && name.indexOf("$") == -1) {
		name = name.substring(0, name.length() - 6);
		load_these.add("org.greenstone.gatherer.msm.parsers." + name);
		DebugStream.println("Loaded metadata parser: " + name);
	    }
	    name = null;
	}
	children = null;
	file = null;
	// Find any metadata parsers found in the jar file (if present)
	File jar_file = new File(Utility.GLI_ARCHIVE);
	if(jar_file.exists()) {
	    try {
		JarFile jar = new JarFile(jar_file);
		for(Enumeration entries = jar.entries(); entries.hasMoreElements(); ) {
		    String name = entries.nextElement().toString();
		    // !! THIS CLASS USES parsers/GreenstoneMetadataParser.java !!
		    if(name.startsWith("org/greenstone/gatherer/msm/parsers/") && name.endsWith(".class") && name.indexOf("$") == -1) {
			name = name.substring(0, name.length() - 6);
			name = name.replace('/', '.');
			if(!load_these.contains(name)) {
			    load_these.add(name);
			    DebugStream.println("Loaded metadata parser: " + name);
			}
		    }						
		    name = null;
		}
		jar = null;
	    }
	    catch (Exception error) {
		error.printStackTrace();
	    }
	}
	jar_file = null;
	// Load create instances of the parsers found.
	for(int i = 0; i < load_these.size(); i++) {
	    try {
		String parser_name = (String) load_these.get(i);
		Class custom_class = Class.forName((String)load_these.get(i));
		MetadataParser custom_parser = (MetadataParser)custom_class.newInstance();
		parsers.add(custom_parser);
		custom_parser = null;
		custom_class = null;
		parser_name = null;
	    }
	    catch (Exception error) {
		error.printStackTrace();
	    }
	}
    }
    /** Locates and assigns metadata for the given file by sequentially applying all active metadata parsers. */
    public boolean searchForMetadata(FileNode destination, FileNode source, boolean folder_level, boolean dummy_run) {
	boolean dialog_cancelled = false;
	int size = parsers.size();
	for(int i = 0; !dialog_cancelled && i < size; i++) {
	    MetadataParser parser = (MetadataParser) parsers.get(i);
	    dialog_cancelled = parser.process(destination, source, folder_level, dummy_run);
	    parser = null;
	}
	return dialog_cancelled;
    }
}
